@@ -85,6 +85,15 @@ sub language_tag {__PACKAGE__->SUPER::language_tag}
  "= Stream this song" => "= ĿǰԴʽ",
  "= Sort by field" => "= ĳλ",
 
+ # Playlist.pm
+ "Add to Playlist" => "벥嵥",
+ "Add All to Playlist" => "ȫ벥嵥",
+ "Current Playlist" => "в嵥",
+ "Clear All" => "ȫ",
+ "Clear Selected" => "ѡȡΧ",
+ "Playlist contains [quant,_1,song,songs]." => "嵥й [quant,_1,,] ",
+ "Your playlist is now full. No more songs can be added." => "嵥޷",
+
 );
 
 1;
